@@ -13,6 +13,7 @@ over a queue means repeatedly calling :meth:`get <Queue.get>` until
 :meth:`get <Queue.get>` returns ``StopIteration`` (specifically that
 class, not an instance or subclass).
 
+    >>> import gevent.queue
     >>> queue = gevent.queue.Queue()
     >>> queue.put(1)
     >>> queue.put(2)
@@ -25,36 +26,52 @@ class, not an instance or subclass).
 .. versionchanged:: 1.0
        ``Queue(0)`` now means queue of infinite size, not a channel. A :exc:`DeprecationWarning`
        will be issued with this argument.
+
+.. versionchanged:: 25.4.1
+   :class:`Queue` was renamed to :class:`SimpleQueue`, while :class:`JoinableQueue` was
+   renamed to :class:`Queue` (`JoinableQueue` remains a backwards compatible alias).
+   This adds the ability to ``join()`` all queues, like the standard library.
+
+   Previously ``SimpleQueue`` was an alias for the undocumented Python
+   implementation ``queue._PySimpleQueue``; now it is gevent's own implementation.
+   This ensures that it is cooperative even without monkey-patching.
 """
 
-from __future__ import absolute_import
+
 import sys
 from heapq import heappush as _heappush
 from heapq import heappop as _heappop
 from heapq import heapify as _heapify
 import collections
+import types
 
-if sys.version_info[0] == 2:
-    import Queue as __queue__ # python 3: pylint:disable=import-error
-else:
-    import queue as __queue__ # python 2: pylint:disable=import-error
-Full = __queue__.Full
-Empty = __queue__.Empty
+import queue as __queue__
+# We re-export these exceptions to client modules.
+# But we also want fast access to them from Cython with a cdef,
+# and we do that with the _ definition.
+_Full = Full = __queue__.Full
+_Empty = Empty = __queue__.Empty
 
 from gevent.timeout import Timeout
 from gevent._hub_local import get_hub_noargs as get_hub
-from greenlet import getcurrent
 from gevent.exceptions import InvalidSwitchError
 
 __all__ = []
-__implements__ = ['Queue', 'PriorityQueue', 'LifoQueue']
+__implements__ = ['Queue', 'PriorityQueue', 'LifoQueue', 'SimpleQueue']
 __extensions__ = ['JoinableQueue', 'Channel']
 __imports__ = ['Empty', 'Full']
-if hasattr(__queue__, 'SimpleQueue'):
-    __all__.append('SimpleQueue') # New in 3.7
-    # SimpleQueue is implemented in C and directly allocates locks
-    # unaffected by monkey patching. We need the Python version.
-    SimpleQueue = __queue__._PySimpleQueue # pylint:disable=no-member
+
+if hasattr(__queue__, 'ShutDown'): # New in 3.13
+    ShutDown = __queue__.ShutDown
+    __imports__.append('ShutDown')
+else:
+    class ShutDown(Exception):
+        """
+        gevent extension for Python versions less than 3.13
+        """
+    __extensions__.append('ShutDown')
+
+
 __all__ += (__implements__ + __extensions__ + __imports__)
 
 
@@ -71,6 +88,8 @@ def _safe_remove(deq, item):
 
 import gevent._waiter
 locals()['Waiter'] = gevent._waiter.Waiter
+locals()['getcurrent'] = __import__('greenlet').getcurrent
+locals()['greenlet_init'] = lambda: None
 
 class ItemWaiter(Waiter): # pylint:disable=undefined-variable
     # pylint:disable=assigning-non-slot
@@ -90,7 +109,8 @@ class ItemWaiter(Waiter): # pylint:disable=undefined-variable
         self.item = None
         return self.switch(self)
 
-class Queue(object):
+
+class SimpleQueue(object):
     """
     Create a queue object with a given maximum size.
 
@@ -109,6 +129,14 @@ class Queue(object):
        an empty queue will now receive items in the order in which they blocked. An
        implementation quirk under CPython *usually* ensured this was roughly the case
        previously anyway, but that wasn't the case for PyPy.
+    .. versionchanged:: 24.10.1
+       Implement the ``shutdown`` methods from Python 3.13.
+    .. versionchanged:: 25.4.1
+       Renamed from ``Queue`` to ``SimpleQueue`` to better match the standard library.
+       While this class no longer has a ``shutdown`` method, the new ``Queue`` class
+       (previously ``JoinableQueue``) continues to have it.
+    .. versionchanged:: 25.4.2
+       Make this class subscriptable.
     """
 
     __slots__ = (
@@ -119,8 +147,11 @@ class Queue(object):
         '_event_unlock',
         'queue',
         '__weakref__',
+        'is_shutdown', # 3.13
     )
 
+    __class_getitem__ = classmethod(types.GenericAlias)
+
     def __init__(self, maxsize=None, items=(), _warn_depth=2):
         if maxsize is not None and maxsize <= 0:
             if maxsize == 0:
@@ -149,6 +180,7 @@ class Queue(object):
         self.hub = get_hub()
         self._event_unlock = None
         self.queue = self._create_queue(items)
+        self.is_shutdown = False
 
     @property
     def maxsize(self):
@@ -223,12 +255,6 @@ class Queue(object):
         """
         return True
 
-    def __nonzero__(self):
-        # Py2.
-        # For Cython; __bool__ becomes a special method that we can't
-        # get by name.
-        return True
-
     def empty(self):
         """Return ``True`` if the queue is empty, ``False`` otherwise."""
         return not self.qsize()
@@ -241,7 +267,8 @@ class Queue(object):
         return self._maxsize > 0 and self.qsize() >= self._maxsize
 
     def put(self, item, block=True, timeout=None):
-        """Put an item into the queue.
+        """
+        Put an item into the queue.
 
         If optional arg *block* is true and *timeout* is ``None`` (the default),
         block if necessary until a free slot is available. If *timeout* is
@@ -250,13 +277,24 @@ class Queue(object):
         Otherwise (*block* is false), put an item on the queue if a free slot
         is immediately available, else raise the :class:`Full` exception (*timeout*
         is ignored in that case).
+
+        ... versionchanged:: 24.10.1
+           Now raises a ``ValueError`` for a negative *timeout* in the cases
+           that CPython does.
         """
+        if self.is_shutdown:
+            raise ShutDown
         if self._maxsize == -1 or self.qsize() < self._maxsize:
-            # there's a free slot, put an item right away
+            # there's a free slot, put an item right away.
+            # For compatibility with CPython, verify that the timeout is non-negative.
+            if block and timeout is not None and timeout < 0:
+                raise ValueError("'timeout' must be a non-negative number")
             self._put(item)
             if self.getters:
                 self._schedule_unlock()
-        elif self.hub is getcurrent():
+            return
+
+        if self.hub is getcurrent(): # pylint:disable=undefined-variable
             # We're in the mainloop, so we cannot wait; we can switch to other greenlets though.
             # Check if possible to get a free slot in the queue.
             while self.getters and self.qsize() and self.qsize() >= self._maxsize:
@@ -266,7 +304,8 @@ class Queue(object):
                 self._put(item)
                 return
             raise Full
-        elif block:
+
+        if block:
             waiter = ItemWaiter(item, self)
             self.putters.append(waiter)
             timeout = Timeout._start_new_or_dummy(timeout, Full)
@@ -279,8 +318,9 @@ class Queue(object):
             finally:
                 timeout.cancel()
                 _safe_remove(self.putters, waiter)
-        else:
-            raise Full
+            return
+
+        raise Full
 
     def put_nowait(self, item):
         """Put an item into the queue without blocking.
@@ -296,7 +336,12 @@ class Queue(object):
         # called from self.peek(). Call this after the initial check
         # to see if there are items in the queue.
 
-        if self.hub is getcurrent():
+        if self.is_shutdown:
+            raise ShutDown
+
+        if block and timeout is not None and timeout < 0:
+            raise ValueError("'timeout' must be a non-negative number")
+        if self.hub is getcurrent(): # pylint:disable=undefined-variable
             # special case to make get_nowait() or peek_nowait() runnable in the mainloop greenlet
             # there are no items in the queue; try to fix the situation by unlocking putters
             while self.putters:
@@ -305,12 +350,19 @@ class Queue(object):
                 self.putters.popleft().put_and_switch()
                 if self.qsize():
                     return method()
-            raise Empty()
+            raise Empty
 
         if not block:
             # We can't block, we're not the hub, and we have nothing
-            # to return. No choice...
-            raise Empty()
+            # to return. No choice but to raise the Empty exception.
+            #
+            # CAUTION: Calling ``q.get(False)`` in a tight loop won't
+            # work like it does in CPython where it should eventually
+            # let another thread make progress, because there's never
+            # a chance to switch greenlets here. We don't sleep()
+            # to enforce that, as that would be a significant behaviour
+            # change.
+            raise Empty
 
         waiter = Waiter() # pylint:disable=undefined-variable
         timeout = Timeout._start_new_or_dummy(timeout, Empty)
@@ -327,7 +379,8 @@ class Queue(object):
             _safe_remove(self.getters, waiter)
 
     def get(self, block=True, timeout=None):
-        """Remove and return an item from the queue.
+        """
+        Remove and return an item from the queue.
 
         If optional args *block* is true and *timeout* is ``None`` (the default),
         block if necessary until an item is available. If *timeout* is a positive number,
@@ -362,7 +415,8 @@ class Queue(object):
         (*timeout* is ignored in that case).
         """
         if self.qsize():
-            # XXX: Why doesn't this schedule an unlock like get() does?
+            # This doesn't schedule an unlock like get() does because we're not
+            # actually making any space.
             return self._peek()
 
         return self.__get_or_peek(self._peek, block, timeout)
@@ -407,8 +461,131 @@ class Queue(object):
             raise result
         return result
 
-    next = __next__ # Py2
 
+class Queue(SimpleQueue):
+    """
+    A subclass of :class:`SimpleQueue` that additionally has
+    :meth:`task_done` and :meth:`join` methods.
+
+    .. versionchanged:: 25.4.1
+       Renamed from ``JoinablQueue`` to simply ``Queue`` to better
+       match the capability of the standard library :class:`queue.Queue`.
+    """
+
+    __slots__ = (
+        '_cond',
+        'unfinished_tasks',
+    )
+
+    def __init__(self, maxsize=None, items=(), unfinished_tasks=None):
+        """
+
+        .. versionchanged:: 1.1a1
+           If *unfinished_tasks* is not given, then all the given *items*
+           (if any) will be considered unfinished.
+
+        """
+        SimpleQueue.__init__(self, maxsize, items, _warn_depth=3)
+
+        from gevent.event import Event
+        self._cond = Event()
+        self._cond.set()
+
+        if unfinished_tasks:
+            self.unfinished_tasks = unfinished_tasks
+        elif items:
+            self.unfinished_tasks = len(items)
+        else:
+            self.unfinished_tasks = 0
+
+        if self.unfinished_tasks:
+            self._cond.clear()
+
+    def copy(self):
+        return type(self)(self.maxsize, self.queue, self.unfinished_tasks)
+
+    def _format(self):
+        result = SimpleQueue._format(self)
+        if self.unfinished_tasks:
+            result += ' tasks=%s _cond=%s' % (self.unfinished_tasks, self._cond)
+        return result
+
+    def _put(self, item):
+        SimpleQueue._put(self, item)
+        self._did_put_task()
+
+    def _did_put_task(self):
+        self.unfinished_tasks += 1
+        self._cond.clear()
+
+    def task_done(self):
+        '''Indicate that a formerly enqueued task is complete. Used by queue consumer threads.
+        For each :meth:`get <Queue.get>` used to fetch a task, a subsequent call to
+        :meth:`task_done` tells the queue that the processing on the task is complete.
+
+        If a :meth:`join` is currently blocking, it will resume when all items have been processed
+        (meaning that a :meth:`task_done` call was received for every item that had been
+        :meth:`put <Queue.put>` into the queue).
+
+        Raises a :exc:`ValueError` if called more times than there were items placed in the queue.
+        '''
+        if self.unfinished_tasks <= 0:
+            raise ValueError('task_done() called too many times')
+        self.unfinished_tasks -= 1
+        if self.unfinished_tasks == 0:
+            self._cond.set()
+
+    def join(self, timeout=None):
+        '''
+        Block until all items in the queue have been gotten and processed.
+
+        The count of unfinished tasks goes up whenever an item is added to the queue.
+        The count goes down whenever a consumer thread calls :meth:`task_done` to indicate
+        that the item was retrieved and all work on it is complete. When the count of
+        unfinished tasks drops to zero, :meth:`join` unblocks.
+
+        :param float timeout: If not ``None``, then wait no more than this time in seconds
+            for all tasks to finish.
+        :return: ``True`` if all tasks have finished; if ``timeout`` was given and expired before
+            all tasks finished, ``False``.
+
+        .. versionchanged:: 1.1a1
+           Add the *timeout* parameter.
+        '''
+        return self._cond.wait(timeout=timeout)
+
+    def shutdown(self, immediate=False):
+        """
+        "Shut-down the queue, making queue gets and puts raise
+        `ShutDown`.
+
+        By default, gets will only raise once the queue is empty. Set
+        *immediate* to True to make gets raise immediately instead.
+
+        All blocked callers of `put` and `get` will be unblocked.
+
+        In joinable queues, if *immediate*, a task is marked as done
+        for each item remaining in the queue, which may unblock
+        callers of `join`.
+        """
+        self.is_shutdown = True
+        if immediate:
+            self._drain_for_immediate_shutdown()
+        getters = list(self.getters)
+        putters = list(self.putters)
+        self.getters.clear()
+        self.putters.clear()
+        for waiter in getters + putters:
+            self.hub.loop.run_callback(waiter.throw, ShutDown)
+
+    def _drain_for_immediate_shutdown(self):
+        while self.qsize():
+            self.get()
+            self.task_done()
+
+#: .. versionchanged:: 25.4.1
+#:  Now a BWC alias
+JoinableQueue = Queue
 
 class UnboundQueue(Queue):
     # A specialization of Queue that knows it can never
@@ -448,14 +625,20 @@ class PriorityQueue(Queue):
 
     def _put(self, item):
         _heappush(self.queue, item)
+        self._did_put_task()
 
     def _get(self):
         return _heappop(self.queue)
 
 
 class LifoQueue(Queue):
-    '''A subclass of :class:`Queue` that retrieves most recently added entries first.'''
+    """
+    A subclass of :class:`JoinableQueue` that retrieves most recently added entries first.
 
+    .. versionchanged:: 24.10.1
+       Now extends :class:`JoinableQueue` instead of just :class:`Queue`.
+
+    """
     __slots__ = ()
 
     def _create_queue(self, items=()):
@@ -463,6 +646,7 @@ class LifoQueue(Queue):
 
     def _put(self, item):
         self.queue.append(item)
+        self._did_put_task()
 
     def _get(self):
         return self.queue.pop()
@@ -471,94 +655,18 @@ class LifoQueue(Queue):
         return self.queue[-1]
 
 
-class JoinableQueue(Queue):
+class Channel:
     """
-    A subclass of :class:`Queue` that additionally has
-    :meth:`task_done` and :meth:`join` methods.
+    A queue-like object that can only hold one item at a
+    time.
+
+    This is commonly used as a synchronization primitive,
+    and is implemented efficiently for this use-case.
+
+    .. versionchanged:: 25.4.2
+       Make this class subscriptable.
     """
 
-    __slots__ = (
-        '_cond',
-        'unfinished_tasks',
-    )
-
-    def __init__(self, maxsize=None, items=(), unfinished_tasks=None):
-        """
-
-        .. versionchanged:: 1.1a1
-           If *unfinished_tasks* is not given, then all the given *items*
-           (if any) will be considered unfinished.
-
-        """
-        Queue.__init__(self, maxsize, items, _warn_depth=3)
-
-        from gevent.event import Event
-        self._cond = Event()
-        self._cond.set()
-
-        if unfinished_tasks:
-            self.unfinished_tasks = unfinished_tasks
-        elif items:
-            self.unfinished_tasks = len(items)
-        else:
-            self.unfinished_tasks = 0
-
-        if self.unfinished_tasks:
-            self._cond.clear()
-
-    def copy(self):
-        return type(self)(self.maxsize, self.queue, self.unfinished_tasks)
-
-    def _format(self):
-        result = Queue._format(self)
-        if self.unfinished_tasks:
-            result += ' tasks=%s _cond=%s' % (self.unfinished_tasks, self._cond)
-        return result
-
-    def _put(self, item):
-        Queue._put(self, item)
-        self.unfinished_tasks += 1
-        self._cond.clear()
-
-    def task_done(self):
-        '''Indicate that a formerly enqueued task is complete. Used by queue consumer threads.
-        For each :meth:`get <Queue.get>` used to fetch a task, a subsequent call to :meth:`task_done` tells the queue
-        that the processing on the task is complete.
-
-        If a :meth:`join` is currently blocking, it will resume when all items have been processed
-        (meaning that a :meth:`task_done` call was received for every item that had been
-        :meth:`put <Queue.put>` into the queue).
-
-        Raises a :exc:`ValueError` if called more times than there were items placed in the queue.
-        '''
-        if self.unfinished_tasks <= 0:
-            raise ValueError('task_done() called too many times')
-        self.unfinished_tasks -= 1
-        if self.unfinished_tasks == 0:
-            self._cond.set()
-
-    def join(self, timeout=None):
-        '''
-        Block until all items in the queue have been gotten and processed.
-
-        The count of unfinished tasks goes up whenever an item is added to the queue.
-        The count goes down whenever a consumer thread calls :meth:`task_done` to indicate
-        that the item was retrieved and all work on it is complete. When the count of
-        unfinished tasks drops to zero, :meth:`join` unblocks.
-
-        :param float timeout: If not ``None``, then wait no more than this time in seconds
-            for all tasks to finish.
-        :return: ``True`` if all tasks have finished; if ``timeout`` was given and expired before
-            all tasks finished, ``False``.
-
-        .. versionchanged:: 1.1a1
-           Add the *timeout* parameter.
-        '''
-        return self._cond.wait(timeout=timeout)
-
-
-class Channel(object):
-
     __slots__ = (
         'getters',
         'putters',
@@ -567,6 +675,8 @@ class Channel(object):
         '__weakref__',
     )
 
+    __class_getitem__ = classmethod(types.GenericAlias)
+
     def __init__(self, maxsize=1):
         # We take maxsize to simplify certain kinds of code
         if maxsize != 1:
@@ -604,7 +714,7 @@ class Channel(object):
         return True
 
     def put(self, item, block=True, timeout=None):
-        if self.hub is getcurrent():
+        if self.hub is getcurrent(): # pylint:disable=undefined-variable
             if self.getters:
                 getter = self.getters.popleft()
                 getter.switch(item)
@@ -634,7 +744,7 @@ class Channel(object):
         self.put(item, False)
 
     def get(self, block=True, timeout=None):
-        if self.hub is getcurrent():
+        if self.hub is getcurrent(): # pylint:disable=undefined-variable
             if self.putters:
                 item, putter = self.putters.popleft()
                 self.hub.loop.run_callback(putter.switch, putter)
@@ -681,5 +791,11 @@ class Channel(object):
 
     next = __next__ # Py2
 
+def _init():
+    greenlet_init() # pylint:disable=undefined-variable
+
+_init()
+
+
 from gevent._util import import_c_accel
 import_c_accel(globals(), 'gevent._queue')
